/*
 * rtesif.c - RTE SIF support code.
 *
 * Copyright (c) 2003-2004 Marcus R. Brown <mrbrown@0xd6.org>
 *
 * Licensed under the AFL v2.0. See the file LICENSE included with this
 * distribution for licensing terms.
 */

#include "types.h"
#include "defs.h"

#define SBIOS_BASE	0x80001000
#define SBIOS_MAGIC	0x80001004
#define SBIOS_MAGICVAL	0x62325350

static int (*_sbios)(int, void *) = NULL;

/*
 * Sony "nop'd" a bunch of undocumented (w.r.t the Linux kernel source) SBIOS
 * calls related to the SIF.  They can be called directly under the Beta kit
 * (version 0x200) but require a direct pointer in the 1.0 kit.  They will
 * not work under any other RTE (e.g. the BBN).
 */
static int sbversion = 0;

static inline int sbios(int func, void *arg)
{
	if (!_sbios) {
		if (_lw(SBIOS_MAGIC) == SBIOS_MAGICVAL)
			_sbios = *(int (**)(int, void *))(SBIOS_BASE);
		else
			return -1;
	}
	return _sbios(func, arg);
}

static int (*_sb_sif_reg_set)(int, int) = NULL;

int sif_reg_set(int reg, int val)
{
	struct setreg_arg {
		int	reg;
		int	val;
	} arg;

	if (sbversion <= 0x200) {
		arg.reg = reg;
		arg.val = val;
		/* sceSifSetReg() */
		return (int)_sbios(21, &arg);
	} else if (sbversion <= 0x250) {
		(int (**)(int, int))_sb_sif_reg_set = 0x80002700;
		return _sb_sif_reg_set(reg, val);
	}

	return 0;
}

static int (*_sb_sif_reg_get)(int) = NULL;

int sif_reg_get(int reg)
{
	if (sbversion <= 0x200) {
		/* sceSifGetReg() */
		return (int)_sbios(22, &reg);
	} else if (sbversion <= 0x250) {
		(int (**)(int))_sb_sif_reg_get = 0x800027c0;
		return _sb_sif_reg_get(reg);
	}

	return 0;
}

int sif_dma_request(void *dmareq, int count)
{
	struct setdma_arg {
		void	*dmareq;
		int	count;
	} arg;

	arg.dmareq = dmareq;
	arg.count  = count;

	/* sceSifSetDma() */
	return _sbios(18, &arg);
}

void sif_init()
{
	sbversion = sbios(0, NULL);

	/* Call the SBIOS SIF init routine */
	_sbios(16, NULL);
}
